#include <Arduino.h>
#include "Pinger.h"

char buffer[1024];

// instruments
const unsigned NUM_INSTRUMENTS = 1;
nm::Instrument* instruments[NUM_INSTRUMENTS];

// pinger
const unsigned PINGER_SIZE = 12;
const unsigned PINGER_PINS[PINGER_SIZE] = {23, 22, 21, 20, 19, 18, 2, 3, 4, 5, 6, 7};

void onNoteOn(byte channel, byte note, byte velocity)
{
  for (unsigned i = 0; i < NUM_INSTRUMENTS; ++i)
  {
    instruments[i]->noteOn(channel, note, velocity);
  }
}

void onNoteOff(byte channel, byte note, byte velocity)
{
  for (unsigned i = 0; i < NUM_INSTRUMENTS; ++i)
  {
    instruments[i]->noteOff(channel, note);
  }
}

void setup()
{
  // set up solenoid instruments first so that they
  // don't stay high for long
  instruments[0] = new nm::Pinger(1, PINGER_PINS, PINGER_SIZE);

  // start serial
  Serial.begin(115200);

  // sensors
  usbMIDI.setHandleNoteOff(onNoteOff);
  usbMIDI.setHandleNoteOn(onNoteOn);
}

void loop()
{
  usbMIDI.read();

  for (unsigned i = 0; i < NUM_INSTRUMENTS; ++i)
  {
    instruments[i]->update();
  }

  if (unsigned numBytes = Serial.available())
  {
    Serial.readBytes(buffer, numBytes);
    unsigned idx = buffer[0] - 'a';
    if (idx >= 0 && idx < PINGER_SIZE)
    {
      Serial.print("Fired solenoid ");
      Serial.print(idx);
      Serial.print(" pin ");
      Serial.println(PINGER_PINS[idx]);
      digitalWrite(PINGER_PINS[idx], HIGH);
      delay(20);
      digitalWrite(PINGER_PINS[idx], LOW);
    }
  }
}
